# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/bifrost_bridge',
                'doc_host': 'https://$GIT_USER_NAME.github.io',
                'git_url': 'https://github.com/$GIT_USER_NAME/bifrost_bridge',
                'lib_path': 'bifrost_bridge'},
  'syms': { 'bifrost_bridge.amrfinderplus': { 'bifrost_bridge.amrfinderplus.process_amrfinderplus_data': ( 'amrfinderplus.html#process_amrfinderplus_data',
                                                                                                           'bifrost_bridge/amrfinderplus.py'),
                                              'bifrost_bridge.amrfinderplus.process_amrfinderplus_data_from_cli': ( 'amrfinderplus.html#process_amrfinderplus_data_from_cli',
                                                                                                                    'bifrost_bridge/amrfinderplus.py')},
            'bifrost_bridge.bifrost': { 'bifrost_bridge.bifrost.process_qc_data': ( 'bifrost.html#process_qc_data',
                                                                                    'bifrost_bridge/bifrost.py')},
            'bifrost_bridge.bracken': { 'bifrost_bridge.bracken.process_bracken_data': ( 'bracken.html#process_bracken_data',
                                                                                         'bifrost_bridge/bracken.py'),
                                        'bifrost_bridge.bracken.process_bracken_data_from_cli': ( 'bracken.html#process_bracken_data_from_cli',
                                                                                                  'bifrost_bridge/bracken.py')},
            'bifrost_bridge.cli': {'bifrost_bridge.cli.cli': ('cli.html#cli', 'bifrost_bridge/cli.py')},
            'bifrost_bridge.core': { 'bifrost_bridge.core.DataFrame': ('core.html#dataframe', 'bifrost_bridge/core.py'),
                                     'bifrost_bridge.core.DataFrame.__init__': ('core.html#dataframe.__init__', 'bifrost_bridge/core.py'),
                                     'bifrost_bridge.core.add_column': ('core.html#add_column', 'bifrost_bridge/core.py'),
                                     'bifrost_bridge.core.cli': ('core.html#cli', 'bifrost_bridge/core.py'),
                                     'bifrost_bridge.core.export_data': ('core.html#export_data', 'bifrost_bridge/core.py'),
                                     'bifrost_bridge.core.export_nested_json_data': ( 'core.html#export_nested_json_data',
                                                                                      'bifrost_bridge/core.py'),
                                     'bifrost_bridge.core.export_nested_xml_data': ( 'core.html#export_nested_xml_data',
                                                                                     'bifrost_bridge/core.py'),
                                     'bifrost_bridge.core.filter_columns': ('core.html#filter_columns', 'bifrost_bridge/core.py'),
                                     'bifrost_bridge.core.filter_rows': ('core.html#filter_rows', 'bifrost_bridge/core.py'),
                                     'bifrost_bridge.core.get_config': ('core.html#get_config', 'bifrost_bridge/core.py'),
                                     'bifrost_bridge.core.get_samplesheet': ('core.html#get_samplesheet', 'bifrost_bridge/core.py'),
                                     'bifrost_bridge.core.hello_world': ('core.html#hello_world', 'bifrost_bridge/core.py'),
                                     'bifrost_bridge.core.import_data': ('core.html#import_data', 'bifrost_bridge/core.py'),
                                     'bifrost_bridge.core.import_nested_json_data': ( 'core.html#import_nested_json_data',
                                                                                      'bifrost_bridge/core.py'),
                                     'bifrost_bridge.core.import_nested_xml_data': ( 'core.html#import_nested_xml_data',
                                                                                     'bifrost_bridge/core.py'),
                                     'bifrost_bridge.core.print_header': ('core.html#print_header', 'bifrost_bridge/core.py'),
                                     'bifrost_bridge.core.rename_header': ('core.html#rename_header', 'bifrost_bridge/core.py'),
                                     'bifrost_bridge.core.set_env_variables': ('core.html#set_env_variables', 'bifrost_bridge/core.py'),
                                     'bifrost_bridge.core.show': ('core.html#show', 'bifrost_bridge/core.py'),
                                     'bifrost_bridge.core.show_project_env_vars': ( 'core.html#show_project_env_vars',
                                                                                    'bifrost_bridge/core.py')},
            'bifrost_bridge.fastp': { 'bifrost_bridge.fastp.process_fastp_data': ( 'fastp.html#process_fastp_data',
                                                                                   'bifrost_bridge/fastp.py'),
                                      'bifrost_bridge.fastp.process_fastp_data_from_cli': ( 'fastp.html#process_fastp_data_from_cli',
                                                                                            'bifrost_bridge/fastp.py')},
            'bifrost_bridge.mlst': { 'bifrost_bridge.mlst.check_if_mlst_empty': ('mlst.html#check_if_mlst_empty', 'bifrost_bridge/mlst.py'),
                                     'bifrost_bridge.mlst.process_mlst_data': ('mlst.html#process_mlst_data', 'bifrost_bridge/mlst.py'),
                                     'bifrost_bridge.mlst.process_mlst_data_from_cli': ( 'mlst.html#process_mlst_data_from_cli',
                                                                                         'bifrost_bridge/mlst.py')},
            'bifrost_bridge.plasmidfinder': { 'bifrost_bridge.plasmidfinder.process_plasmidfinder_data': ( 'plasmidfinder.html#process_plasmidfinder_data',
                                                                                                           'bifrost_bridge/plasmidfinder.py'),
                                              'bifrost_bridge.plasmidfinder.process_plasmidfinder_data_from_cli': ( 'plasmidfinder.html#process_plasmidfinder_data_from_cli',
                                                                                                                    'bifrost_bridge/plasmidfinder.py')},
            'bifrost_bridge.pmlst': { 'bifrost_bridge.pmlst.process_pmlst_data': ( 'pmlst.html#process_pmlst_data',
                                                                                   'bifrost_bridge/pmlst.py'),
                                      'bifrost_bridge.pmlst.process_pmlst_data_from_cli': ( 'pmlst.html#process_pmlst_data_from_cli',
                                                                                            'bifrost_bridge/pmlst.py')},
            'bifrost_bridge.quast': { 'bifrost_bridge.quast.process_quast_data': ( 'quast.html#process_quast_data',
                                                                                   'bifrost_bridge/quast.py'),
                                      'bifrost_bridge.quast.process_quast_data_from_cli': ( 'quast.html#process_quast_data_from_cli',
                                                                                            'bifrost_bridge/quast.py')},
            'bifrost_bridge.rmlst': { 'bifrost_bridge.rmlst.process_rmlst_data': ( 'rmlst.html#process_rmlst_data',
                                                                                   'bifrost_bridge/rmlst.py'),
                                      'bifrost_bridge.rmlst.process_rmlst_data_from_cli': ( 'rmlst.html#process_rmlst_data_from_cli',
                                                                                            'bifrost_bridge/rmlst.py')},
            'bifrost_bridge.shovill': { 'bifrost_bridge.shovill.process_shovill_data': ( 'shovill.html#process_shovill_data',
                                                                                         'bifrost_bridge/shovill.py'),
                                        'bifrost_bridge.shovill.process_shovill_data_from_cli': ( 'shovill.html#process_shovill_data_from_cli',
                                                                                                  'bifrost_bridge/shovill.py')},
            'bifrost_bridge.ssiamb': { 'bifrost_bridge.ssiamb.process_ssiamb_data': ( 'ssiamb.html#process_ssiamb_data',
                                                                                      'bifrost_bridge/ssiamb.py'),
                                       'bifrost_bridge.ssiamb.process_ssiamb_data_from_cli': ( 'ssiamb.html#process_ssiamb_data_from_cli',
                                                                                               'bifrost_bridge/ssiamb.py')}}}
